/********************************************************************
 * Try / Catch / Throw support
 *
 * ---------------------------------------------------------------------
 * Exception Handler structure
 *
 *      An ErrExceptionType object is created for each ErrTry & ErrCatch block.
 *      At any point in the program, there is a linked list of
 *      ErrExceptionType objects. GErrFirstException points to the
 *      most recently entered block. A ErrExceptionType blocks stores
 *      information about the state of the machine (register values)
 *      at the start of the Try block
 ********************************************************************/
typedef long* ErrJumpBuf[12];      // D3-D7,PC,A2-A7

// Structure used to store Try state.
typedef struct ErrExceptionType {
  struct ErrExceptionType* nextP;  // next exception type
  ErrJumpBuf               state;  // setjmp/longjmp storage
  Long                     err;    // Error code
} ErrExceptionType;
typedef ErrExceptionType* ErrExceptionPtr;


/********************************************************************
 * Error Manager Routines
 ********************************************************************/
Int ErrSetJump(ErrJumpBuf buf)
    SYS_TRAP(sysTrapErrSetJump);

void ErrLongJump(ErrJumpBuf buf, Int result)
     SYS_TRAP(sysTrapErrLongJump);

Ptr* ErrExceptionList(void)
     SYS_TRAP(sysTrapErrExceptionList);

void ErrThrow(Long err)
     SYS_TRAP(sysTrapErrThrow);

void ErrDisplayFileLineMsg(CharPtr filename, UInt lineno, CharPtr msg)
     SYS_TRAP(sysTrapErrDisplayFileLineMsg);
