#define noFocus 0xffff

// Update code send as part of a frmUpdate event.
#define frmRedrawUpdateCode                     0x8000


// Alert constants and structures
enum alertTypes {
        informationAlert,
        confirmationAlert,
        warningAlert,
        errorAlert };

typedef enum alertTypes AlertType;

typedef struct {
  Word alertType;
  Word helpRscID;
  Word numButtons;
  Word defaultButton;
} AlertTemplateType;


// Types of object in a dialog box
enum formObjects {
        frmFieldObj,
        frmControlObj,
        frmListObj,
        frmTableObj,
        frmBitmapObj,
        frmLineObj,
        frmFrameObj,
        frmRectangleObj,
        frmLabelObj,
        frmTitleObj,
        frmPopupObj,
        frmGraffitiStateObj,
        frmGadgetObj,
        frmScrollBarObj };

typedef enum formObjects FormObjectKind;


typedef struct {
  Word usable :1; // Set if part of ui
} FormObjAttrType;


typedef struct {
  FormObjAttrType attr;
  PointType       pos;
  Word            rscID;
} FormBitmapType;


typedef struct {
  FormObjAttrType attr;
  PointType       point1;
  PointType       point2;
} FormLineType;


typedef struct {
  Word            id;
  FormObjAttrType attr;
  RectangleType   rect;
  Word            frameType;
} FormFrameType;


typedef struct {
  FormObjAttrType attr;
  RectangleType   rect;
} FormRectangleType;


typedef struct {
  Word            id;
  PointType       pos;
  FormObjAttrType attr;
  FontID          fontID;
  Char *          text;
} FormLabelType;


typedef struct {
  RectangleType rect;
  Char *        text;
} FormTitleType;


typedef struct {
  Word controlID;
  Word listID;
} FormPopupType;


typedef struct {
  PointType pos;
} FrmGraffitiStateType;


typedef struct {
  Word            id;
  FormObjAttrType attr;
  RectangleType   rect;
  VoidPtr         data;
} FormGadgetType;


typedef union {
  void *                 ptr;
  FieldType *            field;
  ControlType *          control;
  ListType        *      list;
  TableType *            table;
  FormBitmapType *       bitmap;
//FormLineType *         line;
//FormFrameType *        frame;
//FormRectangleType *    rectangle;
  FormLabelType *        label;
  FormTitleType *        title;
  FormPopupType *        popup;
  FrmGraffitiStateType * grfState;
  FormGadgetType *       gadget;
  ScrollBarType *        scrollBar;
} FormObjectType;

// typedef FormObjectType * FormObjectPtr;


typedef struct {
  FormObjectKind objectType;
  FormObjectType object;
} FormObjListType;


typedef struct {
  Word usable        :1;   // Set if part of ui
  Word enabled       :1;   // Set if interactable (not grayed out)
  Word visible       :1;   // Set if drawn, used internally
  Word dirty         :1;   // Set if dialog has been modified
  Word saveBehind    :1;   // Set if bits behind form are save when form ids drawn
  Word graffitiShift :1;   // Set if graffiti shift indicator is supported
  Word reserved      :11;
} FormAttrType;


typedef Boolean FormEventHandlerType (EventPtr eventP);

typedef FormEventHandlerType * FormEventHandlerPtr;

typedef struct {
  WindowType          window;
  Word                formId;
  FormAttrType        attr;
  WinHandle           bitsBehindForm;
  FormEventHandlerPtr handler;
  Word                focus;
  Word                defaultButton;
  Word                helpRscId;
  Word                menuRscId;
  Word                numObjects;
  FormObjListType *   objects;
} FormType;


typedef FormType * FormPtr;


//--------------------------------------------------------------------
//
// Form Function
//
//--------------------------------------------------------------------

FormPtr FrmInitForm (Word rscID)
        SYS_TRAP(sysTrapFrmInitForm);

void FrmDeleteForm (FormPtr frm)
     SYS_TRAP(sysTrapFrmDeleteForm);

void FrmDrawForm (FormPtr frm)
     SYS_TRAP(sysTrapFrmDrawForm);

void FrmEraseForm (FormPtr frm)
     SYS_TRAP(sysTrapFrmEraseForm);

FormPtr FrmGetActiveForm (void)
        SYS_TRAP(sysTrapFrmGetActiveForm);

void FrmSetActiveForm (FormPtr frm)
     SYS_TRAP(sysTrapFrmSetActiveForm);

Word FrmGetActiveFormID (void)
     SYS_TRAP(sysTrapFrmGetActiveFormID);

Boolean FrmGetUserModifiedState (FormPtr frm)
        SYS_TRAP(sysTrapFrmGetUserModifiedState);

void FrmSetNotUserModified (FormPtr frm)
     SYS_TRAP(sysTrapFrmSetNotUserModified);

Word FrmGetFocus (FormPtr frm)
     SYS_TRAP(sysTrapFrmGetFocus);

void FrmSetFocus (FormPtr frm, Word fieldIndex)
     SYS_TRAP(sysTrapFrmSetFocus);

Boolean FrmHandleEvent (FormPtr frm, EventPtr eventP)
        SYS_TRAP(sysTrapFrmHandleEvent);

void FrmGetFormBounds (FormPtr frm, RectanglePtr r)
     SYS_TRAP(sysTrapFrmGetFormBounds);

WinHandle FrmGetWindowHandle (FormPtr frm)
          SYS_TRAP(sysTrapFrmGetWindowHandle);

Word FrmGetFormId (FormPtr frm)
     SYS_TRAP(sysTrapFrmGetFormId);

FormPtr FrmGetFormPtr (Word formId)
        SYS_TRAP(sysTrapFrmGetFormPtr);

FormPtr FrmGetFirstForm (void)
        SYS_TRAP(sysTrapFrmGetFirstForm);

Word FrmGetNumberOfObjects (FormPtr frm)
     SYS_TRAP(sysTrapFrmGetNumberOfObjects);

Word FrmGetObjectIndex (FormPtr frm, Word objID)
     SYS_TRAP(sysTrapFrmGetObjectIndex);

Word FrmGetObjectId (FormPtr, Word objIndex)
     SYS_TRAP(sysTrapFrmGetObjectId);

FormObjectKind FrmGetObjectType (FormPtr frm, Word objIndex)
               SYS_TRAP(sysTrapFrmGetObjectType);

VoidPtr FrmGetObjectPtr (FormPtr frm, Word objIndex)
        SYS_TRAP(sysTrapFrmGetObjectPtr);

void FrmGetObjectBounds (FormPtr frm, Word pObjIndex, RectanglePtr r)
     SYS_TRAP(sysTrapFrmGetObjectBounds);

void FrmHideObject (FormPtr frm, Word objIndex)
     SYS_TRAP(sysTrapFrmHideObject);

void FrmShowObject (FormPtr frm, Word objIndex)
     SYS_TRAP(sysTrapFrmShowObject);

void FrmGetObjectPosition (FormPtr frm, Word objIndex,
                           SWordPtr x, SWordPtr y)
     SYS_TRAP(sysTrapFrmGetObjectPosition);

void FrmSetObjectPosition (FormPtr frm, Word objIndex,
                           SWord x, SWord y)
     SYS_TRAP(sysTrapFrmSetObjectPosition);

void FrmSetObjectBounds (FormPtr frm, Word objIndex,
                         RectanglePtr bounds)
     SYS_TRAP(sysTrapFrmSetObjectBounds);


SWord FrmGetControlValue (FormPtr frm, Word controlID)
      SYS_TRAP(sysTrapFrmGetControlValue);

void FrmSetControlValue (FormPtr frm, Word controlID, SWord newValue)
     SYS_TRAP(sysTrapFrmSetControlValue);

Byte FrmGetControlGroupSelection (FormPtr frm, Byte groupNum)
     SYS_TRAP(sysTrapFrmGetControlGroupSelection);

void FrmSetControlGroupSelection (FormPtr frm, Byte groupNum,
                                  Word controlID)
     SYS_TRAP(sysTrapFrmSetControlGroupSelection);

void FrmCopyLabel (FormPtr frm, Word lableID, CharPtr newLable)
     SYS_TRAP(sysTrapFrmCopyLabel);

CharPtr FrmGetLabel (FormPtr frm, Word lableID)
        SYS_TRAP(sysTrapFrmGetLabel);

void FrmSetCategoryLabel (FormPtr frm, Word objIndex, CharPtr newLabel)
     SYS_TRAP(sysTrapFrmSetCategoryLabel);

CharPtr FrmGetTitle (FormPtr frm)
        SYS_TRAP(sysTrapFrmGetTitle);

void FrmSetTitle (FormPtr frm, CharPtr newTitle)
     SYS_TRAP(sysTrapFrmSetTitle);

void FrmCopyTitle (FormPtr frm, CharPtr newTitle)
     SYS_TRAP(sysTrapFrmCopyTitle);

VoidPtr FrmGetGadgetData (FormPtr frm, Word objIndex)
        SYS_TRAP(sysTrapFrmGetGadgetData);

void FrmSetGadgetData (FormPtr frm, Word objIndex, VoidPtr data)
     SYS_TRAP(sysTrapFrmSetGadgetData);

Word FrmDoDialog (FormPtr frm)
     SYS_TRAP(sysTrapFrmDoDialog);

Word FrmAlert (Word  alertId)
     SYS_TRAP(sysTrapFrmAlert);

Word FrmCustomAlert (Word  alertId, CharPtr s1, CharPtr s2, CharPtr s3)
     SYS_TRAP(sysTrapFrmCustomAlert);

void FrmHelp (Word helpMsgId)
     SYS_TRAP(sysTrapFrmHelp);

void FrmUpdateScrollers (FormPtr frm, Word upIndex, Word downIndex,
                         Boolean scrollableUp, Boolean scrollabledown)
     SYS_TRAP(sysTrapFrmUpdateScrollers);

Boolean FrmVisible (FormPtr frm)
        SYS_TRAP(sysTrapFrmVisible);

void FrmSetEventHandler (FormPtr frm, FormEventHandlerPtr)
     SYS_TRAP(sysTrapFrmSetEventHandler);

Boolean FrmDispatchEvent (EventPtr eventP)
        SYS_TRAP(sysTrapFrmDispatchEvent);


void FrmPopupForm (Word formId)
     SYS_TRAP(sysTrapFrmPopupForm);

void FrmGotoForm (Word formId)
     SYS_TRAP(sysTrapFrmGotoForm);

void FrmUpdateForm (Word formId, Word updateCode)
     SYS_TRAP(sysTrapFrmUpdateForm);

void FrmReturnToForm  (Word formId)
     SYS_TRAP(sysTrapFrmReturnToForm);

void FrmCloseAllForms (void)
     SYS_TRAP(sysTrapFrmCloseAllForms);

void FrmSaveAllForms (void)
     SYS_TRAP(sysTrapFrmSaveAllForms);

Boolean FrmPointInTitle (FormPtr frm, Short x, Short y)
        SYS_TRAP(sysTrapFrmPointInTitle);

void FrmSetMenu (FormPtr frm, Word menuRscID)
     SYS_TRAP(sysTrapFrmSetMenu);
