#define noListSelection                 0x0ffff

//-------------------------------------------------------------------
// List structures
//-------------------------------------------------------------------

typedef struct {
  Word usable         :1; // set if part of ui
  Word enabled        :1; // set if interactable (not grayed out)
  Word visible        :1; // set if drawn
  Word poppedUp       :1; // set if choices displayed in popup win.
  Word hasScrollBar   :1; // set if the list has a scroll bar
  Word search         :1; // set if incremental search is enabled
  Word reserved       :2;
} ListAttrType;


// Load data callback routine prototype
typedef void ListDrawDataFuncType (Word itemNum, RectanglePtr bounds,
        CharPtr *itemsText);

typedef ListDrawDataFuncType * ListDrawDataFuncPtr;


typedef struct {
  Word                 id;
  RectangleType        bounds;
  ListAttrType         attr;
  CharPtr              *itemsText;
  Word                 numItems;          // number of choices in the list
  Word                 currentItem;       // currently display choice
  Word                 topItem;           // top item visible when poped up
  FontID               font;              // font used to draw list
  WinHandle            popupWin;          // used only by popup lists
  ListDrawDataFuncPtr  drawItemsCallback; // 0 indicates no function
} ListType;

typedef ListType * ListPtr;


//-------------------------------------------------------------------
// List routines
//-------------------------------------------------------------------
void LstDrawList (ListPtr list)
     SYS_TRAP(sysTrapLstDrawList);

void LstEraseList (ListPtr list)
     SYS_TRAP(sysTrapLstEraseList);

Word LstGetSelection (ListPtr list)
     SYS_TRAP(sysTrapLstGetSelection);

CharPtr LstGetSelectionText (ListPtr list, Word itemNum)
        SYS_TRAP(sysTrapLstGetSelectionText);

Boolean LstHandleEvent (ListPtr list, EventPtr event)
        SYS_TRAP(sysTrapLstHandleEvent);

void LstSetHeight (ListPtr pList, Word  visibleItems)
     SYS_TRAP(sysTrapLstSetHeight);

void LstSetPosition (ListPtr pList, SWord x, SWord y)
     SYS_TRAP(sysTrapLstSetPosition);

void LstSetSelection (ListPtr list, Word itemNum)
     SYS_TRAP(sysTrapLstSetSelection);

void LstSetListChoices (ListPtr list, Char ** itemsText,
                        Word numItems)
     SYS_TRAP(sysTrapLstSetListChoices);

void LstSetDrawFunction (ListPtr pList, ListDrawDataFuncPtr func)
     SYS_TRAP(sysTrapLstSetDrawFunction);

void LstSetTopItem (ListPtr pList, Word itemNum)
     SYS_TRAP(sysTrapLstSetTopItem);

void LstMakeItemVisible (ListPtr pList, Word itemNum)
     SYS_TRAP(sysTrapLstMakeItemVisible);

Word LstGetNumberOfItems (ListPtr pList)
     SYS_TRAP(sysTrapLstGetNumberOfItems);

SWord LstPopupList (ListPtr list)
      SYS_TRAP(sysTrapLstPopupList);

Boolean LstScrollList(ListPtr pList, enum directions direction, short itemCount)
        SYS_TRAP(sysTrapLstScrollList);

Int LstGetVisibleItems (ListPtr pList)
    SYS_TRAP(sysTrapLstGetVisibleItems);
