enum directions { up = 0, down, left, right };
typedef enum directions DirectionType;


enum windowFormats { screenFormat = 0, genericFormat };
typedef enum windowFormats WindowFormatType;


//enum graphicModes { drawMode = 0, eraseMode, invertMode, mergeMode};
//typedef enum graphicModes GraphicModeType;
typedef enum {scrCopy, scrAND, scrANDNOT, scrXOR, scrOR, scrCopyNOT} ScrOperation;


enum patterns { blackPattern, whitePattern, grayPattern, customPattern };
typedef enum patterns PatternType;

#define grayHLinePattern        0xAA

enum underlineModes { noUnderline, grayUnderline, solidUnderline };
typedef enum underlineModes UnderlineModeType;



typedef struct {
  Word compressed:1;      // Data format:  0=raw; 1=compressed
  Word reserved :15;
} BitmapFlagsType;


typedef struct {
  Word            width;
  Word            height;
  Word            rowBytes;
  BitmapFlagsType flags;
  Word            reserved [4];
} BitmapType;



typedef BitmapType * BitmapPtr;

typedef Word CustomPatternType [4];

typedef struct {
  ScrOperation      grafMode;
  Boolean           patternMode;
  PatternType       pattern;
  CustomPatternType customPtn;
  FontID            fontId;
  Byte              padding1;
  FontPtr           font;
  UnderlineModeType underlineMode;
} GraphicStateType;

typedef GraphicStateType * GraphicStatePtr;


typedef union {
        struct {
                Word cornerDiam   : 8;            // corner diameter, max 38
                Word reserved     : 4;
                Word shadowWidth  : 2;            // Width of shadow
                Word width        : 2;            // Width frame
        } bits;
        Word word;
} FrameBitsType;

typedef Word FrameType;

//  Standard Frame Types
#define noFrame         0
#define simpleFrame     1
#define rectangleFrame  1
#define roundFrame      0x0401          // corner = 7, frame = 1
#define boldRoundFrame  0x0702          // corner = 7, frame = 2
#define popupFrame      0x0205          // corner = 2,  frame = 1, shadow = 1
#define dialogFrame     0x0302          // corner = 3,  frame = 2
#define menuFrame       popupFrame


typedef struct {
        Word format:1;      // window format:  0=screen mode; 1=generic mode
        Word offscreen:1;   // offscreen flag: 0=onscreen ; 1=offscreen
        Word modal:1;       // modal flag:     0=modeless window; 1=modal window
        Word focusable:1;   // focusable flag: 0=non-focusable; 1=focusable
        Word enabled:1;     // enabled flag:   0=disabled; 1=enabled
        Word visible:1;     // visible flag:   0-invisible; 1=visible
        Word dialog:1;      // dialog flag:    0=non-dialog; 1=dialog
        Word compressed:1;  // compressed flag:  0=uncompressed; 1=compressed
        Word reserved :8;
} WindowFlagsType;


typedef struct WinTypeStruct {
  Word                  displayWidth;
  Word                  displayHeight;
  VoidPtr               displayAddr;
  WindowFlagsType       windowFlags;
  RectangleType         windowBounds;
  AbsRectType           clippingBounds;
  PointType             viewOrigin;
  FrameBitsType         frameType;
  GraphicStatePtr       gstate;
  struct WinTypeStruct *nextWindow;
} WindowType;

typedef WindowType * WinPtr;
typedef WinPtr       WinHandle;





//-----------------------------------------------
//  Macros
//-----------------------------------------------

// For now, the window handle is a pointer to a window structure,
// this however may change, so use the following macros.

#define WinGetWindowPointer(winHandle) ((WinPtr) winHandle)

#define WinGetWindowHandle(winPtr) ((WinHandle) winPtr)


//-----------------------------------------------
// Routines relating to windows management
//-----------------------------------------------

WinHandle WinCreateWindow (RectanglePtr bounds, FrameType frame,
                           Boolean modal, Boolean focusable, WordPtr error)
          SYS_TRAP(sysTrapWinCreateWindow);

WinHandle WinCreateOffscreenWindow (SWord width, SWord height,
                                    WindowFormatType format, WordPtr error)
          SYS_TRAP(sysTrapWinCreateOffscreenWindow);

void WinDeleteWindow (WinHandle winHandle, Boolean eraseIt)
     SYS_TRAP(sysTrapWinDeleteWindow);

void WinInitializeWindow (WinHandle winHandle)
     SYS_TRAP(sysTrapWinInitializeWindow);

void WinAddWindow (WinHandle winHandle)
     SYS_TRAP(sysTrapWinAddWindow);

void WinRemoveWindow (WinHandle winHandle)
     SYS_TRAP(sysTrapWinRemoveWindow);

void WinSetActiveWindow (WinHandle winHandle)
     SYS_TRAP(sysTrapWinSetActiveWindow);

WinHandle WinSetDrawWindow (WinHandle winHandle)
          SYS_TRAP(sysTrapWinSetDrawWindow);

WinHandle WinGetDrawWindow (void)
          SYS_TRAP(sysTrapWinGetDrawWindow);

WinHandle WinGetActiveWindow (void)
          SYS_TRAP(sysTrapWinGetActiveWindow);

WinHandle WinGetDisplayWindow (void)
          SYS_TRAP(sysTrapWinGetDisplayWindow);

WinHandle WinGetFirstWindow (void)
          SYS_TRAP(sysTrapWinGetFirstWindow);

void WinEnableWindow (WinHandle winHandle)
     SYS_TRAP(sysTrapWinEnableWindow);

void WinDisableWindow (WinHandle winHandle)
     SYS_TRAP(sysTrapWinDisableWindow);

void WinGetWindowFrameRect (WinHandle winHandle, RectanglePtr r)
     SYS_TRAP(sysTrapWinGetWindowFrameRect);

void WinDrawWindowFrame (void)
     SYS_TRAP(sysTrapWinDrawWindowFrame);

void WinEraseWindow (void)
     SYS_TRAP(sysTrapWinEraseWindow);

WinHandle WinSaveBits (RectanglePtr source, WordPtr error)
          SYS_TRAP(sysTrapWinSaveBits);

void WinRestoreBits (WinHandle winHandle, SWord destX, SWord destY)
     SYS_TRAP(sysTrapWinRestoreBits);

void WinCopyRectangle (WinHandle srcWin, WinHandle dstWin,
                       RectanglePtr srcRect, SWord destX, SWord destY, ScrOperation mode)
     SYS_TRAP(sysTrapWinCopyRectangle);

void WinScrollRectangle(RectanglePtr r, enum directions direction, SWord distance,
                        RectanglePtr vacated)
     SYS_TRAP(sysTrapWinScrollRectangle);

void WinGetDisplayExtent (SWordPtr extentX, SWordPtr extentY)
     SYS_TRAP(sysTrapWinGetDisplayExtent);

void WinGetWindowBounds (RectanglePtr r)
     SYS_TRAP(sysTrapWinGetWindowBounds);

void WinSetWindowBounds (WinHandle winHandle, RectanglePtr r)
     SYS_TRAP(sysTrapWinSetWindowBounds);

void WinGetWindowExtent (SWordPtr extentX, SWordPtr extentY)
     SYS_TRAP(sysTrapWinGetWindowExtent);

void WinDisplayToWindowPt (SWordPtr extentX, SWordPtr extentY)
     SYS_TRAP(sysTrapWinDisplayToWindowPt);

void WinWindowToDisplayPt (SWordPtr extentX, SWordPtr extentY)
     SYS_TRAP(sysTrapWinWindowToDisplayPt);

void WinGetClip (RectanglePtr r)
     SYS_TRAP(sysTrapWinGetClip);

void WinSetClip (RectanglePtr r)
     SYS_TRAP(sysTrapWinSetClip);

void WinResetClip (void)
     SYS_TRAP(sysTrapWinResetClip);

void WinClipRectangle (RectanglePtr r)
     SYS_TRAP(sysTrapWinClipRectangle);

Boolean WinModal (WinHandle winHandle)
        SYS_TRAP(sysTrapWinModal);

//-----------------------------------------------
// Routines to draw shapes or frames shapes
//-----------------------------------------------

void WinDrawLine (SWord x1, SWord y1, SWord x2, SWord y2)
     SYS_TRAP(sysTrapWinDrawLine);

void WinDrawGrayLine (SWord x1, SWord y1, SWord x2, SWord y2)
     SYS_TRAP(sysTrapWinDrawGrayLine);

void WinEraseLine (SWord x1, SWord y1, SWord x2, SWord y2)
     SYS_TRAP(sysTrapWinEraseLine);

void WinInvertLine (SWord x1, SWord y1, SWord x2, SWord y2)
     SYS_TRAP(sysTrapWinInvertLine);

void WinFillLine (SWord x1, SWord y1, SWord x2, SWord y2)
     SYS_TRAP(sysTrapWinFillLine);

void WinDrawRectangle (RectanglePtr r, Word cornerDiam)
     SYS_TRAP(sysTrapWinDrawRectangle);

void WinEraseRectangle (RectanglePtr r, Word cornerDiam)
     SYS_TRAP(sysTrapWinEraseRectangle);

void WinInvertRectangle (RectanglePtr r, Word cornerDiam)
     SYS_TRAP(sysTrapWinInvertRectangle);

void WinFillRectangle (RectanglePtr r, Word cornerDiam)
     SYS_TRAP(sysTrapWinFillRectangle);

void WinDrawRectangleFrame (FrameType frame, RectanglePtr r)
     SYS_TRAP(sysTrapWinDrawRectangleFrame);

void WinDrawGrayRectangleFrame (FrameType frame, RectanglePtr r)
     SYS_TRAP(sysTrapWinDrawGrayRectangleFrame);

void WinEraseRectangleFrame (FrameType frame, RectanglePtr r)
     SYS_TRAP(sysTrapWinEraseRectangleFrame);

void WinInvertRectangleFrame (FrameType frame, RectanglePtr r)
     SYS_TRAP(sysTrapWinInvertRectangleFrame);

void WinGetFramesRectangle (FrameType  frame, RectanglePtr r,
                            RectanglePtr obscuredRect)
     SYS_TRAP(sysTrapWinGetFramesRectangle);



//-----------------------------------------------
// Routines relating to characters
//-----------------------------------------------

void WinDrawChars (CharPtr chars, Word len, SWord x, SWord y)
     SYS_TRAP(sysTrapWinDrawChars);

void WinDrawInvertedChars (CharPtr chars, Word len, SWord x, SWord y)
     SYS_TRAP(sysTrapWinDrawInvertedChars);

void WinEraseChars (CharPtr chars, Word len, SWord x, SWord y)
     SYS_TRAP(sysTrapWinEraseChars);

void WinInvertChars (CharPtr chars, Word len, SWord x, SWord y)
     SYS_TRAP(sysTrapWinInvertChars);

UnderlineModeType WinSetUnderlineMode (UnderlineModeType mode)
                  SYS_TRAP(sysTrapWinSetUnderlineMode);


//-----------------------------------------------
// Routines to control patterns
//-----------------------------------------------

void WinGetPattern (CustomPatternType)
     SYS_TRAP(sysTrapWinGetPattern);

void WinSetPattern (CustomPatternType)
     SYS_TRAP(sysTrapWinSetPattern);

//-----------------------------------------------
// Routines relating to bitmaps
//-----------------------------------------------
void WinDrawBitmap (BitmapPtr bitmapP, SWord x, SWord y)
     SYS_TRAP(sysTrapWinDrawBitmap);
